# -*- coding: utf-8 -*-
import re, json, datetime
from functools import reduce
from hashlib import md5
import time
try:
    from urllib2 import urlopen, Request
except ImportError:
    from urllib.request import urlopen, Request
try:
    from urllib import quote, urlencode
except ImportError:
    from urllib.parse import quote, urlencode
try:
    from urlparse import urlparse, parse_qs, urljoin
except ImportError:
    from urllib.parse import urlparse, parse_qs, urljoin
ua = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/113.0.0.0 Safari/537.36'
buvid3 = ''
img_key = ''
sub_key = ''
mixinKeyEncTab = [
    46, 47, 18, 2, 53, 8, 23, 32, 15, 50, 10, 31, 58, 3, 45, 35, 27, 43, 5, 49,
    33, 9, 42, 19, 29, 28, 14, 39, 12, 38, 41, 13, 37, 48, 7, 16, 24, 55, 40,
    61, 26, 17, 0, 1, 60, 51, 30, 4, 22, 25, 54, 21, 56, 59, 6, 63, 57, 62, 11,
    36, 20, 34, 44, 52
]

def getjson(url):
    global buvid3
    if not buvid3:
        req = Request('https://www.bilibili.com/', headers={'User-Agent': ua})
        res = urlopen(req)
        cookie = res.info().get('Set-Cookie')
        if cookie:
            buvid3 = re.search(r'buvid3\s*=\s*(.+?);', cookie)
            if buvid3:
                buvid3 = buvid3.group(1)
    req = Request(url, headers={'Cookie': 'buvid3=%s' % buvid3, 'User-Agent': ua})
    return json.loads(urlopen(req).read().decode('utf-8'))


def getMixinKey(orig):
    '对 imgKey 和 subKey 进行字符顺序打乱编码'
    return reduce(lambda s, i: s + orig[i], mixinKeyEncTab, '')[:32]


def encWbi(params, img_key, sub_key):
    '为请求参数进行 wbi 签名'
    mixin_key = getMixinKey(img_key + sub_key)
    curr_time = int(time.time())
    params['wts'] = curr_time                                   # 添加 wts 字段
    # 过滤 value 中的 "!'()*" 字符
    params = [
        (k, ''.join(filter(lambda chr: chr not in "!'()*", str(v))))
        for k, v 
        in sorted(params.items())                 # 按照 key 重排参数
    ]
    query = urlencode(params)                      # 序列化参数
    wbi_sign = md5((query + mixin_key).encode()).hexdigest()    # 计算 w_rid
    params.append(('w_rid', wbi_sign))
    return params


def getWbiKeys():
    '获取最新的 img_key 和 sub_key'
    json_content = getjson('https://api.bilibili.com/x/web-interface/nav')
    img_url = json_content['data']['wbi_img']['img_url']
    sub_url = json_content['data']['wbi_img']['sub_url']
    img_key = img_url.rsplit('/', 1)[1].split('.')[0]
    sub_key = sub_url.rsplit('/', 1)[1].split('.')[0]
    return img_key, sub_key


def getWbi(params):
    global img_key, sub_key
    if not img_key or not sub_key:
        img_key, sub_key = getWbiKeys()
    return encWbi(
        params=params,
        img_key=img_key,
        sub_key=sub_key
    )
    

def search(s, t, page=1):
    if not s:
        return {'code': 'bilibili 300'}
    if t == 'video':
        t = 'video'
        k = 'search_video'
    elif t == 'space':
        t = 'bili_user'
        k = 'search_space'
    else:
        return {'code': 'bilibili 200'}
    url = 'https://api.bilibili.com/x/web-interface/search/type?keyword=%s&search_type=%s&page=%d' % (quote(s), t, page)
    try:
        data = getjson(url)
    except Exception as e:
        return {'code': 'bilibili 201', 'error': str(e)}
    ret = []
    for e in data['data']['result']:
        if e['type'] == 'video':
            title = e['title']
            title = title.replace('<em class="keyword">', '')
            title = title.replace('</em>', '')
            img = e['pic']
            url = 'https://www.bilibili.com/video/%s' % e['bvid']
            plot = []
            plot.append(e['author'])
            plot.append(datetime.datetime.utcfromtimestamp(e['pubdate']).strftime('%Y-%m-%d %H:%M:%S'))
            plot.append(e['description'])
        if e['type'] == 'bili_user':
            title = e['uname']
            img = e['upic']
            url = 'https://space.bilibili.com/%d/video' % e['mid']
            plot = []
            plot.append(u'稿件: %d' % e['videos'])
            plot.append(u'粉丝: %d' % e['fans'])
        plot = '\n'.join(plot)
        ret.append({'title': title, 'url': url, 'img': urljoin(url, img), 'plot': plot})
    numPages = int(data['data']['numPages'])
    if page < numPages:
        ret.append({'title': 'Next Page (%d / %d)' % (page + 1, numPages), k: s, 'page': page + 1})
    return ret


def videolist(url):
    o = urlparse(url)
    p = o.path.split('/')
    q = parse_qs(o.query, True)
    url1 = 'https://api.bilibili.com/x/player/pagelist?bvid=%s' % p[-1]
    if 'p' in q:
        return ykdl(url)
    try:
        data = getjson(url1)
    except Exception as e:
        return {'code': 'bilibili 301', 'error': str(e)}
    if len(data['data']) == 1:
        return ykdl(url)
    ret = []
    for e in data['data']:
        ret.append({'title': '%d. %s' % (e['page'], e['part']), 'url': 'https://www.bilibili.com/video/%s?p=%d' % (url, e['page'])})
    return ret


def spacelist(url, page):
    o = urlparse(url)
    p = o.path.split('/')
    q = parse_qs(o.query, True)
    signed_params = getWbi({'mid': p[-2], 'pn': page})
    query = urlencode(signed_params)
    url1 = 'https://api.bilibili.com/x/space/wbi/arc/search?%s' % query
    try:
        data = getjson(url1)
    except Exception as e:
        return {'code': 'bilibili 401', 'error': str(e)}
    ret = []
    for e in data['data']['list']['vlist']:
        title = e['title']
        img = e['pic']
        url2 = 'https://www.bilibili.com/video/%s' % e['bvid']
        plot = []
        plot.append(e['author'])
        plot.append(datetime.datetime.utcfromtimestamp(e['created']).strftime('%Y-%m-%d %H:%M:%S'))
        plot.append(e['description'])
        plot = '\n'.join(plot)
        ret.append({'title': title, 'url': url2, 'img': urljoin(url, img), 'plot': plot})
    total_page = (data['data']['page']['count'] + data['data']['page']['ps'] - 1) // data['data']['page']['ps']
    if page < total_page:
        ret.append({'title': 'Next Page (%d / %d)' % (page + 1, total_page), 'url': url, 'page': page + 1})
    return ret


def ykdl(url):
    from ykdlWrapper import url_to_module
    m, u = url_to_module(url)
    parser = m.parser
    parser(u)
    info = m.prepare()
    info = info.jsonlize()
    ret = []
    for k, v in info['streams'].items():
        ret.append({'title': k, 'url': v['src']})
    return ret
